package com.allyourcode.aidlclient;

import android.app.Activity;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.os.IBinder;
import android.os.RemoteException;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;

import com.allyourcode.aidlservice.IMyAidlInterface;


public class MainActivity extends Activity {

  IMyAidlInterface reporter;
  ServiceConnection connection = new MyServiceConnection();
  SharedPreferences prefs;
  boolean isBound = false;

  void bind() {
    Intent intent = new Intent();
    intent.setClassName("com.allyourcode.aidlservice",
       "com.allyourcode.aidlservice.MyService");
    isBound = bindService(intent, connection,
       Context.BIND_AUTO_CREATE);
  }

  public void queryService() {
    if (isBound) {
      try {
        String report = reporter
           .fetchWeather(locationText.getText()
              .toString());
        textView.setText(report);
      } catch (RemoteException e) {
        e.printStackTrace();
      }
    } else {
      textView.setText(R.string.service_not_bound);
    }
  }

  void unbind() {
    if (isBound) {
      unbindService(connection);
      isBound = false;
    }
  }

  class MyServiceConnection implements ServiceConnection {
    public void onServiceConnected(
       ComponentName className, IBinder binder) {
      reporter = IMyAidlInterface.Stub
         .asInterface(binder);
    }

    public void onServiceDisconnected(ComponentName n) {
    }
  }

  // The rest of the code is copied from
  // Listing 3-7 (from the onDestroy method,
  // downward).

  @Override
  public void onDestroy() {
    super.onDestroy();
    prefs = getSharedPreferences("PREFS", MODE_PRIVATE);
    SharedPreferences.Editor editor = prefs.edit();
    editor.putBoolean("isBound", isBound);
    editor.putString("report", textView.getText()
       .toString());
    editor.commit();

    unbind();
  }

  @Override
  public void onCreate(Bundle savedInstanceState) {
    super.onCreate(savedInstanceState);
    setContentView(R.layout.activity_main);
    prefs = getSharedPreferences("PREFS", MODE_PRIVATE);
    if (prefs != null) {
      textView = (TextView) findViewById(R.id.textView);

      textView.setText(prefs.getString("report",
         getString(R.string.report_appears_here)));
      if (prefs.getBoolean("isBound", false)) {
        bind();
      }
    }

    bindButton = (Button) findViewById(R.id.buttonBind);
    locationText =
       (EditText) findViewById(R.id.editText);
    queryButton = (Button) findViewById(R.id.buttonQuery);
    unbindButton =
       (Button) findViewById(R.id.buttonUnbind);
  }

  public static TextView textView;
  Button bindButton, queryButton, unbindButton;
  EditText locationText;

  public void onButtonClick(View view) {
    if (view == bindButton) {
      bind();
    } else if (view == queryButton) {
      queryService();
    } else if (view == unbindButton) {
      unbind();
    }
  }
}

